from plone import api
from plone.dexterity.content import Container

from seantis.plonetools import tests
from seantis.plonetools.behaviors.customtitle import on_object_modified


class CustomTitleContainer(Container):

    def get_custom_title(self):
        return self.custom_title


class TestBehaviors(tests.IntegrationTestCase):

    def test_custom_title_behavior(self):

        behavior = 'seantis.plonetools.behaviors.customtitle.ICustomTitle'
        klass = 'seantis.plonetools.tests.test_behaviors.CustomTitleContainer'

        portal_type = self.new_temporary_type(
            behaviors=[behavior], klass=klass
        ).id

        with self.user('admin'):
            obj = api.content.create(
                id='',
                type=portal_type,
                container=self.new_temporary_folder(),
                custom_title=u'My custom title'
            )

        self.assertEqual(obj.id, 'my-custom-title')
        self.assertEqual(obj.title, u'My custom title')

        obj.custom_title = u'Another swell title'
        on_object_modified(obj)

        # the id does not change after initial creation
        self.assertEqual(obj.id, 'my-custom-title')
        self.assertEqual(obj.title, u'Another swell title')



from plone import api

from seantis.plonetools import tests
from seantis.plonetools.browser import BaseForm


class TestBaseForm(tests.IntegrationTestCase):

    def test_urls(self):

        form = BaseForm(api.portal.get(), self.request)

        self.assertEqual(form.success_url, 'http://nohost/plone')
        self.assertEqual(form.cancel_url, 'http://nohost/plone')

    def test_translate(self):

        form = BaseForm(api.portal.get(), self.request)

        self.assertRaises(NotImplementedError, form.translate, u'')
        self.assertEqual(form.translate(u'Yes', domain='plone'), u'Yes')

    def test_buttons(self):

        form = BaseForm(api.portal.get(), self.request)
        form.update()

        self.assertEqual(len(form.buttons), 2)
        self.assertEqual(len(form.handlers._handlers), 2)

        self.assertIn('context', form.actions['save'].klass)



from mock import Mock

from seantis.plonetools import async
from seantis.plonetools import tests


class TestAsync(tests.IntegrationTestCase):

    def setUp(self):
        super(TestAsync, self).setUp()
        self.old_logger = async.log
        async.log = Mock()

    def tearDown(self):
        super(TestAsync, self).tearDown()
        async.log = self.old_logger

    def test_register_clock_server(self):
        result = async.register('method', 60 * 60)
        self.assertEquals(result.method, 'method')
        self.assertTrue('method' in async._clockservers)

        async.clear_clockservers()
        self.assertTrue('method' not in async._clockservers)

    def test_clock_logger(self):
        logger = async.ClockLogger('method')

        logger.log('GET http://localhost:888/method HTTP/1.1 200')
        self.assertEquals(0, len(async.log.method_calls))

        logger.log('GET http://localhost:888/method HTTP/1.1 500')
        self.assertTrue('call.warn' in str(async.log.method_calls))
        self.assertTrue('500' in str(async.log.method_calls))

        logger.log('GET http://localhost:888/method HTTP/1.1')
        self.assertTrue('call.error' in str(async.log.method_calls))

        logger.log('')
        self.assertEquals(3, len(async.log.method_calls))

        logger.log('abcd')
        self.assertEquals(4, len(async.log.method_calls))



import logging
log = logging.getLogger('seantis.plonetools')

import colour
import stdnum.ch.ssn
import stdnum.iban

from urlparse import urlparse
from zope.schema import TextLine, URI
from zope.schema.interfaces import InvalidURI

from zope.interface import Invalid

from Products.CMFDefault.utils import checkEmailAddress
from Products.CMFDefault.exceptions import EmailAddressInvalid

from seantis.plonetools import _


class Email(TextLine):

    def __init__(self, *args, **kwargs):
        super(TextLine, self).__init__(*args, **kwargs)

    def _validate(self, value):
        super(TextLine, self)._validate(value)
        validate_email(value)


class Website(URI):
    """URI field which assumes http:// if no protocol is specified."""

    def fromUnicode(self, value):
        try:
            if not urlparse(value).scheme:
                value = u'http://' + value
        except:
            log.exception('invalid url %s' % value)
            raise InvalidURI(value)

        return super(Website, self).fromUnicode(value)


class HexColor(TextLine):

    def __init__(self, *args, **kwargs):
        super(TextLine, self).__init__(*args, **kwargs)

    def _validate(self, value):
        super(TextLine, self)._validate(value)
        validate_hex_color(value)


class IBAN(TextLine):

    def __init__(self, *args, **kwargs):
        super(TextLine, self).__init__(*args, **kwargs)

    def _validate(self, value):
        super(TextLine, self)._validate(value)
        validate_iban(value)


class SwissSocialSecurityNumber(TextLine):

    def __init__(self, *args, **kwargs):
        super(TextLine, self).__init__(*args, **kwargs)

    def _validate(self, value):
        super(TextLine, self)._validate(value)
        validate_swiss_ssn(value)


def validate_email(value):
    try:
        email = (value or u'').strip()
        if email:
            checkEmailAddress(email)
    except EmailAddressInvalid:
        raise Invalid(_(u"Invalid email address: {}".format(email)))

    return True


def validate_hex_color(value):
    try:
        color = (value or u'').strip()
        if color:
            colour.Color(value)
    except (ValueError, AttributeError):
        raise Invalid(_(u"Invalid hex color: {}".format(color)))

    return True


def validate_iban(value):
    iban = (value or u'').strip()

    if iban and not stdnum.iban.is_valid(iban):
        raise Invalid(_(u"Invalid IBAN number: {}".format(iban)))

    return True


def validate_swiss_ssn(value):
    ssn = (value or u'').strip()

    if ssn and not stdnum.ch.ssn.is_valid(ssn):
        raise Invalid(_(u"Invalid Swiss Social Security number: {}".format(
            ssn
        )))

    return True


# optional plone.schemaeditor integration
try:
    from plone.schemaeditor.fields import FieldFactory
    EmailFactory = FieldFactory(Email, _(u"Email"))
    WebsiteFactory = FieldFactory(Website, _(u"Website"))
    HexColorFactory = FieldFactory(HexColor, _(u"Color"))
    IBANFactory = FieldFactory(IBAN, _(u"IBAN"))
    SwissSocialSecurityNumberFactory = FieldFactory(
        SwissSocialSecurityNumber, _(u"Swiss Social Security Number")
    )
except ImportError:
    pass


# optional plone.supermodel integration
try:
    from plone.supermodel.exportimport import BaseHandler
    EmailHandler = BaseHandler(Email)
    WebsiteHandler = BaseHandler(Website)
    HexColorHandler = BaseHandler(HexColor)
    IBANHandler = BaseHandler(IBAN)
    SwissSocialSecurityNumberHandler = BaseHandler(SwissSocialSecurityNumber)
except ImportError:
    pass



from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from zope.configuration import xmlconfig

class TestLayer(PloneSandboxLayer):

    default_bases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):

        import seantis.plonetools
        xmlconfig.file(
            'configure.zcml',
            seantis.plonetools,
            context=configurationContext
        )
        self.loadZCML(package=seantis.plonetools)


TESTFIXTURE = TestLayer()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(TESTFIXTURE, ),
    name="Testfixture:Integration"
)



import logging
from Products.CMFCore.utils import getToolByName


""" DRY version of the following:
http://maurits.vanrees.org/weblog/archive/2009/12/catalog

1. Create your-module.default.txt in your profiles/default

2. Warp the functions in your setuphandler.py, providing your module name
and the list of wanted indexes.

e.g.

from seantis.plonetools import setuphandlers

indexes = [
    ('first_letter', 'FieldIndex')
]


def add_catalog_indexes(context, logger=None):
    setuphandlers.add_catalog_indexes(
        'seantis.people', indexes, context, logger
    )


def import_indexes(context):
    setuphandlers.import_indexes(
        'seantis.people', indexes, context
    )

3. Add your own import step to your configure.zcml

e.g.
    <!-- Import step for indexes -->
    <genericsetup:importStep
        name="seantis.people"
        description=""
        title="seantis.people indexes"
        handler="seantis.people.setuphandler.import_indexes"
    />

"""


def add_catalog_indexes(module, wanted, context, logger=None):
    """Method to add our wanted indexes to the portal_catalog.

    When called from the import_various method below, 'context' is
    the plone site and 'logger' is the portal_setup logger.  But
    this method can also be used as upgrade step, in which case
    'context' will be portal_setup and 'logger' will be None.

    """

    logger = logger or logging.getLogger(module)

    # Run the catalog.xml step as that may have defined new metadata
    # columns.  We could instead add <depends name="catalog"/> to
    # the registration of our import step in zcml, but doing it in
    # code makes this method usable as upgrade step as well.  Note that
    # this silently does nothing when there is no catalog.xml, so it
    # is quite safe.
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(
        'profile-{}:default'.format(module), 'catalog'
    )

    catalog = getToolByName(context, 'portal_catalog')
    indexes = catalog.indexes()

    indexables = []
    for name, meta_type in wanted:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
            indexables.append(name)
            logger.info("Added %s for field %s.", meta_type, name)

    if indexables:
        logger.info("Indexing new indexes %s.", ', '.join(indexables))
        catalog.manage_reindexIndex(ids=indexables)


def import_indexes(module, wanted, context):
    """Import step for configuration that is not handled in xml files.
    """
    # Only run step if a flag file is present
    if context.readDataFile('{}-default.txt'.format(module)) is None:
        return

    site = context.getSite()
    add_catalog_indexes(module, wanted, site, context.getLogger(module))



from zope.security.management import newInteraction, endInteraction
from seantis.plonetools.testlayer import INTEGRATION_TESTING

from seantis.plonetools.testing import TestCase

# to use with integration where security interactions need to be done manually
class IntegrationTestCase(TestCase):
    layer = INTEGRATION_TESTING

    def setUp(self):
        super(IntegrationTestCase, self).setUp()
        # setup security
        newInteraction()

    def tearDown(self):
        endInteraction()
        super(IntegrationTestCase, self).tearDown()



""" Installs the BeakerSessionDataManager into the Zope Root to enable
memcached sessions with collective.beaker and
Products.BeakerSessionDataManager.

Run as follows:

bin/instance run <path to seantis.plonetools>
/seantis/plonetools/scripts/install-memcached-sessions.py

"""


import transaction
import logging
log = logging.getLogger('seantis.plonetools')


def main(app):
    try:
        from Products.BeakerSessionDataManager.sessiondata import (
            addBeakerSessionDataManager, BeakerSessionDataManager
        )
    except ImportError:
        log.error('BeakerSessionDataManager not found.')
        return

    if 'session_data_manager' in app:
        if isinstance(app['session_data_manager'], BeakerSessionDataManager):
            log.warn('BeakerSessionDataManager is already installed.')
            return

        del app['session_data_manager']

    dispatcher = app.manage_addProduct['BeakerSessionDataManager']
    addBeakerSessionDataManager(
        dispatcher, 'session_data_manager', 'Beaker Session Data Manager'
    )
    transaction.commit()

    log.info('Sucessfully installed BeakerSessionDataManager')

if 'app' in locals():
    main(locals()['app'])



# -*- coding: utf-8 -*-

from App.config import getConfiguration, setConfiguration
from plone import api

from zope import i18n
from zope.i18nmessageid import MessageFactory

from Products.ZCatalog.interfaces import ICatalogBrain

from seantis.plonetools import tests
from seantis.plonetools import tools


class TestTools(tests.IntegrationTestCase):

    def test_get_parent(self):

        folder = self.new_temporary_folder(title='parent')
        new_type = self.new_temporary_type()

        with self.user('admin'):
            obj = api.content.create(
                id='test',
                type=new_type.id,
                container=folder
            )

        brain = tools.get_brain_by_object(obj)

        self.assertEqual(tools.get_parent(brain).Title, 'parent')
        self.assertEqual(tools.get_parent(obj).title, 'parent')

        self.assertTrue(ICatalogBrain.providedBy(tools.get_parent(brain)))
        self.assertTrue(isinstance(tools.get_parent(obj), folder.__class__))

    def test_in_debug_mode(self):
        cfg = getConfiguration()

        cfg.debug_mode = False
        setConfiguration(cfg)

        self.assertFalse(tools.in_debug_mode())

        cfg.debug_mode = True
        setConfiguration(cfg)

        self.assertTrue(tools.in_debug_mode())

    def test_is_existing_portal_type(self):
        new_type = self.new_temporary_type()

        self.assertTrue(tools.is_existing_portal_type(new_type.id))
        self.assertFalse(tools.is_existing_portal_type('inexistant'))

    def test_safe_html(self):
        self.assertEqual(
            '<div></div>',
            tools.safe_html('<div><script>alert("x");</script></div>')
        )

    def test_get_type_info_by_behavior(self):
        basic_behavior = 'plone.app.dexterity.behaviors.metadata.IBasic'

        basic_type = self.new_temporary_type(behaviors=[basic_behavior])
        self.new_temporary_type(behaviors=[])

        self.assertEqual(
            tools.get_type_info_by_behavior(basic_behavior), [basic_type]
        )

    def test_get_type_info_by_schema(self):
        model = """<?xml version='1.0' encoding='utf8'?>
        <model xmlns="http://namespaces.plone.org/supermodel/schema">
            <schema>
                <field name="foo" type="zope.schema.TextLine" />
            </schema>
        </model>
        """

        new_type = self.new_temporary_type(model_source=model)
        self.assertEqual(
            tools.get_type_info_by_schema(new_type.lookupSchema()),
            [new_type]
        )

    def test_get_schema_from_portal_type(self):
        new_type = self.new_temporary_type()

        self.assertEqual(
            tools.get_schema_from_portal_type(new_type.id),
            new_type.lookupSchema()
        )

    def test_get_brain_by_object(self):
        with self.user('admin'):
            obj = api.content.create(
                id='test',
                type=self.new_temporary_type().id,
                container=self.new_temporary_folder()
            )

        brain = tools.get_brain_by_object(obj)
        self.assertIs(type(brain.getObject()), type(obj))
        self.assertEqual(brain.id, obj.id)

    def test_invert_dictionary(self):
        input = {
            1: 'x',
            2: 'x',
            3: 'y'
        }
        output = {
            'x': [1, 2],
            'y': [3]
        }
        self.assertEqual(tools.invert_dictionary(input), output)

    def test_add_attribute_to_metadata(self):
        catalog = api.portal.get_tool('portal_catalog')

        tools.add_attribute_to_metadata('test')
        self.assertIn('test', catalog._catalog.schema)

        # ensure that a second call doesn't do anything
        tools.add_attribute_to_metadata('test')
        self.assertIn('test', catalog._catalog.schema)

    def test_order_fields_by_schema(self):

        model = """<?xml version='1.0' encoding='utf8'?>
        <model xmlns="http://namespaces.plone.org/supermodel/schema">
            <schema>
                <field name="first" type="zope.schema.TextLine"></field>
                <field name="second" type="zope.schema.TextLine"></field>
            </schema>
        </model>"""

        schema = self.new_temporary_type(model_source=model).lookupSchema()

        self.assertEqual(
            tools.order_fields_by_schema(
                ['second', 'x', 'last', 'first'], schema
            ),
            ['first', 'second', 'x', 'last']
        )

    def test_translator(self):
        _ = MessageFactory('plone')
        self.assertEqual(
            tools.translator(self.request, domain='plone')(_(u'Comment')),
            i18n.translate(_(u'Comment'), target_language='en', domain='plone')
        )

    def test_unicode_collate_sortkey(self):
        self.assertEqual(
            sorted(u'AaÄäÖöOo', key=tools.unicode_collate_sortkey()),
            list(u'aAäÄoOöÖ')
        )



from five import grok

from z3c.form import button
from z3c.form.interfaces import ActionExecutionError
from plone.z3cform.fieldsets.extensible import ExtensibleForm

from zope.interface import Invalid

from plone.directives.form import Form

from seantis.plonetools import _
from seantis.plonetools.browser.shared import (
    TranslateMixin,
    StatusMessageMixin
)


class BaseForm(ExtensibleForm, Form, TranslateMixin, StatusMessageMixin):
    """ Baseform which should serve as a base for any and all Plone Forms
    by Seantis. It abstracts away some complexity from the underlying z3c forms
    and provides often used methods.

    The goal of this class is to remove duplicated code throughout our
    projects.

    Note, to use fieldsets/widget hints and so forth you need to have a schema
    inheriting from plone.supermodel.model.Schema:

    class ISchema(plone.supermodel.model.Schema)
        pass

    And a form like this:

    class Form(BaseForm, plone.autoform.form.AutoExtensibleForm):

        schema = IMySchema

    """

    grok.baseclass()

    created_msg = _(u"Item created")
    changed_msg = _(u"Changes saved")
    unchanged_msg = _(u"No changes saved")

    def __init__(self, *args, **kwargs):
        super(BaseForm, self).__init__(*args, **kwargs)

        try:
            from plone.directives.dexterity import AddForm
            self.is_add_form = isinstance(self, AddForm)
        except ImportError:
            self.is_add_form = False

    def update(self):
        self.prepare_actions()
        super(BaseForm, self).update()

    def updateActions(self):
        super(BaseForm, self).updateActions()

        # add the css class to the buttons which have a custom css class
        for action in self.available_actions:
            if 'css_class' in action:
                self.actions[action['name']].addClass(action['css_class'])

    @property
    def available_actions(self):
        """ Returns the available actions (buttons) of this form. The default
        style of doing this by using decorators has the disadvantage that
        buttons may not be inherited.

        For example, we always redefine 'Cancel' for all forms.

        By overriding this form and returning a list of dictionaries, the
        buttons can easily be defined. For example:

        return [
            dict(name='delete', title=_(u'Delete'), css_class='destructive'),
            dict(name='cancel', title=_(u'Cancel'))
        ]

        The handlers for these actions are handle_delete and handle_cancel.
        As should be obvious, the function is implicitly defined as
        handle_<action-name>.

        All actions always go through handle_action first, bevore the
        respective functions are called.

        handle_save and handle_cancel are available as default handlers.

        """

        yield dict(name='save', title=_(u'Save'), css_class='context')
        yield dict(name='cancel', title=_(u'Cancel'))

    @property
    def success_url(self):
        """ Redirect used when handle_save runs successfully. """
        return self.context.absolute_url()

    @property
    def cancel_url(self):
        """ Redirect used when the user clicks on handle_cancel. """
        return self.context.absolute_url()

    def prepare_actions(self):
        """ Takes the actions defined in available_actions and adds
        them as buttons and handlers. Note that the css_class is added
        using updateActions.

        """
        self.buttons = button.Buttons()
        self.handlers = button.Handlers()

        for action in self.available_actions:

            btn = button.Button(title=action['title'], name=action['name'])
            self.buttons += button.Buttons(btn)

            button_handler = button.Handler(btn, self.__class__.handle_action)
            self.handlers.addHandler(btn, button_handler)

    def handle_action(self, action):
        """ All actions flow through this handler by default. It checks
        for the availability of handle_<action_name> and calls them
        if they are present.

        """
        button_handler_id = 'handle_{name}'.format(name=action.__name__)
        button_handler_fn = getattr(self.__class__, button_handler_id)

        assert button_handler_fn, """Button {name} expects a button handler
        funnction named 'handle_{name}', this handler could not be found
        on class {cls}.""".format(name=action.__name__, cls=self.__class__)

        return button_handler_fn(self)

    def before_save(self, data):
        """ Called with the validated data before the changes are applied.
        Liast chance to add custom validations.

        """
        pass

    def handle_save(self):
        """ Default handler for saves. Gets the data, applies them and
        shows a message if there were changes. Redirects to self.success_url
        if no validation errors stopped the process.

        """
        data = self.parameters

        if data is None:
            return

        self.before_save(data)

        if self.is_add_form:
            obj = self.createAndAdd(data)
            if obj is not None:
                self._finishedAdd = True
                self.message(self.created_msg)
            else:
                return
        else:
            if self.applyChanges(data):
                self.message(self.changed_msg)
            else:
                self.message(self.unchanged_msg)

        self.request.response.redirect(self.success_url)

    def handle_cancel(self):
        self.request.response.redirect(self.cancel_url)

    @property
    def parameters(self):
        """ Extracts the form data and returns a dictionary or None if
        something went wrong. If something does go wrong, the error message
        is automatically set.

        """
        data, errors = self.extractData()

        if errors:
            self.status = self.formErrorsMessage
            return None
        else:
            return data

    def raise_action_error(self, msg):
        """ Raise the given message as action execution error, which will
        pop up on top of the form.

        """
        raise ActionExecutionError(Invalid(msg))



# -*- coding: utf-8 -*-

from zope.interface import Invalid

from seantis.plonetools import tests
from seantis.plonetools import schemafields


class TestSchemafields(tests.IntegrationTestCase):

    def test_validate_email(self):
        schemafields.validate_email(u'')
        schemafields.validate_email(u'test@example.org')
        schemafields.validate_email(u' test@example.org ')
        self.assertRaises(Invalid, schemafields.validate_email, u'asdf')

    def test_validate_color(self):
        schemafields.validate_hex_color('')
        schemafields.validate_hex_color('red')
        schemafields.validate_hex_color('#000')
        self.assertRaises(Invalid, schemafields.validate_hex_color, u'#0')

    def test_validate_iban(self):
        schemafields.validate_iban(u'')
        schemafields.validate_iban(u'BE31435411161155')
        self.assertRaises(Invalid, schemafields.validate_iban, u'asdf')

    def test_swiss_social_security_number(self):
        schemafields.validate_swiss_ssn(u'')
        schemafields.validate_swiss_ssn(u'756.9217.0769.85')
        schemafields.validate_swiss_ssn(u'7569217076985')
        self.assertRaises(
            Invalid, schemafields.validate_iban, u'756.9217.0769.84'
        )



import unittest2 as unittest
import uuid

from plone.dexterity import fti
from contextlib import contextmanager

from zope import event
from zope.traversing.interfaces import BeforeTraverseEvent

from zope.component import getSiteManager
from Products.CMFPlone.tests.utils import MockMailHost
from Products.MailHost.interfaces import IMailHost

from plone.testing import z2
from plone import api

from collective.betterbrowser import new_browser

from seantis.plonetools import tools


def install_mailhost(portal, mailhost):
    portal.MailHost = mailhost

    sm = getSiteManager(context=portal)
    sm.unregisterUtility(provided=IMailHost)
    sm.registerUtility(mailhost, provided=IMailHost)


def install_mock_mailhost(portal, from_address='noreply@example.com'):
    portal._original_mailhost = portal.MailHost
    portal.email_from_address = from_address

    install_mailhost(portal, MockMailHost('MailHost'))


def uninstall_mock_mailhost(portal):
    if hasattr(portal, '_original_mailhost'):
        install_mailhost(portal, portal._original_mailhost)


class TestCase(unittest.TestCase):
    """ Testcase which can be used with different Seantis plone modules. """

    def setUp(self):

        self.app = self.layer['app']
        self.portal = self.layer['portal']

        self.temporary_folders = []
        self.temporary_types = []

        install_mock_mailhost(self.portal)

        # remove all test event subscribers
        event.subscribers = [
            e for e in event.subscribers if type(e) != TestEventSubscriber
        ]

        self.current_user = None

        # setup manually the correct browserlayer, see:
        # https://dev.plone.org/ticket/11673
        event.notify(BeforeTraverseEvent(self.portal, self.request))

    def tearDown(self):
        uninstall_mock_mailhost(self.portal)
        self.remove_temporary_folders()
        self.remove_temporary_types()
        self.logout()

    @property
    def request(self):
        return self.layer['request']

    @property
    def mailhost(self):
        return self.portal.MailHost

    def login(self, user):
        if user == 'admin':
            acl = self.app['acl_users']
        else:
            acl = self.portal['acl_users']

        z2.login(acl, user)
        self.current_user = user

    def logout(self):
        z2.logout()
        self.current_user = None

    @contextmanager
    def user(self, user):
        """ Use as follows:

        with self.user('admin'):
            # do admin stuff

        """
        old_user = self.current_user
        self.login(user)

        yield

        self.logout()
        if old_user:
            self.login(old_user)

    def new_browser(self):
        return new_browser(self.layer)

    def subscribe(self, eventclass):
        subscriber = TestEventSubscriber(eventclass)
        event.subscribers.append(subscriber)
        return subscriber

    def new_temporary_folder(self, title=None):
        with self.user('admin'):
            folder = api.content.create(
                type='Folder',
                title=title or uuid.uuid4().hex,
                container=self.portal
            )

            self.temporary_folders.append(folder)

            return folder

    def remove_temporary_folders(self):
        with self.user('admin'):
            for folder in self.temporary_folders:
                api.content.delete(obj=folder)

    def new_temporary_type(self, **kwargs):
        """ Creates a new dexterity type and registers it with plone. Use it
        to easily test behaviors.

        """
        with self.user('admin'):
            new_type = tools.add_new_dexterity_type(uuid.uuid4().hex, **kwargs)
            self.temporary_types.append(new_type)

            return new_type

    def remove_temporary_types(self):
        types = api.portal.get_tool('portal_types')

        with self.user('admin'):
            for temp_type in self.temporary_types:
                types._delObject(temp_type.id)

                fti.unregister(temp_type)


class TestEventSubscriber(object):

    def __init__(self, eventclass):
        self.eventclass = eventclass
        self.event = None

    def __call__(self, event):
        if type(event) is self.eventclass:
            self.event = event

    def was_fired(self):
        return self.event is not None

    def reset(self):
        self.event = None



import sys
import os.path
import time

from functools import partial, wraps

from App.config import getConfiguration

from plone import api
from plone.dexterity.fti import (
    register as register_dexterity_type,
    DexterityFTI
)
from plone.dexterity.interfaces import IDexterityFTI
from Products.ZCatalog.interfaces import ICatalogBrain
from Products.PortalTransforms.transforms.safe_html import scrubHTML

from zope.component import getMultiAdapter
from zope.component.hooks import getSite
from zope.component import getUtility, getAllUtilitiesRegisteredFor
from zope.component.interfaces import ComponentLookupError
from zope.schema import getFields
from zope import i18n


import pyuca

allkeys = os.path.join('/'.join(os.path.split(__file__)[:-1]), 'allkeys.txt')
collator = pyuca.Collator(allkeys)


def public(f):
    """ Use a decorator to avoid retyping function/class names.

      * Based on an idea by Duncan Booth:
      http://groups.google.com/group/comp.lang.python/msg/11cbb03e09611b8a
      * Improved via a suggestion by Dave Angel:
      http://groups.google.com/group/comp.lang.python/msg/3d400fb22d8a42e1
    """
    all = sys.modules[f.__module__].__dict__.setdefault('__all__', [])
    if f.__name__ not in all:  # Prevent duplicates if run from an IDE.
        all.append(f.__name__)
    return f


@public
def safe_html(html):
    """ Returns the given html without dangerous elements like <script/>. The
    result will still contain html however, not replacments like '&gt;'.

    """
    return scrubHTML(html, raise_error=False)


@public
def get_parent(obj):
    """ Gets the parent of the obj or brain. The result is of the same type
    as the parameter given.

    """

    if ICatalogBrain.providedBy(obj):
        catalog = api.portal.get_tool('portal_catalog')
        brains = catalog(
            path={'query': '/'.join(obj.getPath().split('/')[:-1])}
        )
        return brains[0]
    else:
        return obj.aq_inner.aq_parent


@public
def is_existing_portal_type(portal_type):
    """ Return True if the given portal_type can be looked up. """

    try:
        getUtility(IDexterityFTI, portal_type)
    except ComponentLookupError:
        return False

    return True


@public
def get_type_info_by_behavior(behavior):
    """ Returns a list of Dexterity ftis with the given behavior enabled. """

    ftis = getAllUtilitiesRegisteredFor(IDexterityFTI)
    return [fti for fti in ftis if behavior in fti.behaviors]


@public
def get_type_info_by_schema(schema):
    """ Returns a list of Dexterity ftis using the given schema. """

    ftis = getAllUtilitiesRegisteredFor(IDexterityFTI)
    return [fti for fti in ftis if schema == fti.lookupSchema()]


@public
def get_schema_from_portal_type(portal_type):
    """ Get the schema from a portal type. """

    fti = getUtility(IDexterityFTI, portal_type)
    return fti.lookupSchema()


@public
def order_fields_by_schema(fields, schema):
    """ Gets the order of the given scheman and orders the given fields by
    that order. If a field is not found in the schema it is added at the end.

    """

    order = dict((key, f.order) for key, f in getFields(schema).items())
    return sorted(fields, key=lambda field: order.get(field, sys.maxint))


@public
def add_attribute_to_metadata(attribute):
    """ Adds the given attribute to the metadata in the portal catalog. If
    the attribute is alredy there function silently skips it.

    """
    zcatalog = api.portal.get_tool('portal_catalog')._catalog

    if attribute not in zcatalog.schema:
        zcatalog.addColumn(attribute)


@public
def get_brain_by_object(obj):
    """ The inverse of getObject. """
    catalog = api.portal.get_tool('portal_catalog')

    return catalog(path={'query': '/'.join(obj.getPhysicalPath())})[0]


@public
def in_debug_mode():
    """ What you think it is. """
    return getConfiguration().debug_mode


@public
def add_new_dexterity_type(name, **kwargs):
    """ Adds a new dexterity type the same way Dexterity does it internally,
    when using the form. The difference is that this function does not
    assume any defaults and creates the types with the parameters given.

    Behaviors may be given as a list (Dexterity needs a multiline string).
    """
    new_type = DexterityFTI(name)

    if not 'klass' in kwargs:
        kwargs['klass'] = 'plone.dexterity.content.Container'

    if 'behaviors' in kwargs:
        if isinstance(kwargs['behaviors'], list):
            kwargs['behaviors'] = '\n'.join(kwargs['behaviors'])

    new_type.manage_changeProperties(
        **kwargs
    )

    types = api.portal.get_tool('portal_types')
    types._setObject(new_type.id, new_type)

    register_dexterity_type(new_type)

    return new_type


@public
def invert_dictionary(dictionary):
    """ Inverts a dictionary as follows:

    {
        1: 'x',
        2: 'x',
        3: 'y'
    }

    ->

    {
        'x': [1, 2],
        'y': [3]
    }

    """
    inverted = {}

    for k, v in dictionary.iteritems():
        inverted[v] = inverted.get(v, [])
        inverted[v].append(k)

    return inverted


@public
def get_current_language(request):
    """ Returns the current language of the request. """
    portal_state = getMultiAdapter(
        (getSite(), request), name=u'plone_portal_state'
    )
    return portal_state.language()


@public
def translator(request, domain=None):
    """ Returns a function which takes a single string and translates it using
    the curried language of the request

    """

    # xx-xx languages will not work here, though they work when Plone does
    # it in a template. For now it does not matter as we have no country
    # specific translation available
    lang = get_current_language(request).split('-')[0]

    return partial(i18n.translate, target_language=lang, domain=domain)


@public
def unicode_collate_sortkey(case_sensitive=True):
    """ Returns a sort function to sanely sort unicode values.

    A more exact solution would be to use pyUCA but that relies on an external
    C Library and is more complicated

    See:
    http://stackoverflow.com
    /questions/1097908/how-do-i-sort-unicode-strings-alphabetically-in-python
    http://en.wikipedia.org/wiki/ISO_14651
    http://unicode.org/reports/tr10/
    http://pypi.python.org/pypi/PyICU
    http://jtauber.com/blog/2006/01/27/python_unicode_collation_algorithm/
    https://github.com/href/Python-Unicode-Collation-Algorithm

    """

    return collator.sort_key


@public
def profile(fn):
    """ Naive profiling of a function.. on unix systems only. """

    @wraps(fn)
    def wrapper(*args, **kwargs):
        start = time.time()

        result = fn(*args, **kwargs)
        print fn.__name__, 'took', (time.time() - start) * 1000, 'ms'

        return result

    return wrapper



from plone.app.content.interfaces import INameFromTitle
from zope.interface import implements, Interface
from zope.component import adapts


class ICustomTitle(Interface):
    pass


class CustomTitle(object):
    """ Calls get_custom_title on a dexterity type's class to get the
    title and id on creation. To use add the following to your dexterity
    type xml:

    <!-- enabled behaviors -->
    <property name="behaviors">
        <element
            value="seantis.plonetools.behaviors.customtitle.ICustomTitle"
        />
    </property>

    And add get_custom_title to your class defined in <property name="klass">:

    def get_custom_title(self):
        return u'My custom title'

    """
    implements(INameFromTitle)
    adapts(ICustomTitle)

    def __init__(self, context):
        pass

    def __new__(cls, context):
        title = context.get_custom_title()
        instance = super(CustomTitle, cls).__new__(cls)

        instance.title = title
        context.setTitle(title)

        return instance


def on_object_modified(obj, event=None):
    obj.setTitle(obj.get_custom_title())
    obj.reindexObject()



from five import grok

from seantis.plonetools.browser.shared import (
    TranslateMixin,
    StatusMessageMixin
)


class BaseView(grok.View, TranslateMixin, StatusMessageMixin):

    grok.baseclass()



from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import (
    newSecurityManager, setSecurityManager
)
from AccessControl.User import UnrestrictedUser as BaseUnrestrictedUser
from contextlib import contextmanager
from plone import api


class UnrestrictedUser(BaseUnrestrictedUser):
    """Unrestricted user that still has an id.
    """
    def getId(self):
        """Return the ID of the user.
        """
        return self.getUserName()


@contextmanager
def run_as(role):
    """ Execute code under special role priviledges.

    For example:

    with run_as('Manager'):
        do_dangerous_things()

    Please note that this is dangerous and should only be used in a very
    limited and well understood fashion. Try to do as little as possible.

    Better yet, find a way around this.
    """

    sm = getSecurityManager()
    portal = api.portal.get()

    try:
        try:
            # Clone the current user and assign a new role.
            # Note that the username (getId()) is left in exception
            # tracebacks in the error_log,
            # so it is an important thing to store.
            tmp_user = UnrestrictedUser(
                sm.getUser().getId(), '', [role], ''
            )

            # Wrap the user in the acquisition context of the portal
            tmp_user = tmp_user.__of__(portal.acl_users)
            newSecurityManager(None, tmp_user)

            yield

        except:
            # If special exception handlers are needed, run them here
            raise
    finally:
        # Restore the old security manager
        setSecurityManager(sm)



import re
from copy import copy
from z3c.form import group, field


spaces = r'\s'


class BaseGroup(group.Group):
    """ A generic group to use with form groups. Groups are preferrable to
    formsets on the mainform as the access to fields and widgets is much
    clearer. For example:

    class GeneralGroup(BaseGroup):
        label = _(u'General')
        group_fields = [
            [IEvent, ['start', 'end']
        ]

    class Form(BaseForm):
        groups = (GeneralGroup, )
        enable_form_tabbing = True

    You can also add more than one interface to the group_fields. For example:

    group_fields = [
        [IEvent, ['start', 'end']]
        [ILocation, ['lat', 'long']]
    ]

    Note that in this case you might have to write your own handle_save code
    in the form. The default handle_save code will only work if only one
    Interface is used (in one or over many groups).

    """

    @property
    def __name__(self):
        return re.sub(spaces, u'_', self.label.lower())

    @property
    def group_fields(self):
        raise NotImplementedError

    @property
    def fields(self):
        """ Returns the fields defined in group_fields, shallow copying all
        fields before they are returned so they can be safely manipulated.

        It is important that they are copied since these changes otherwise
        leak through to other forms.

        """

        if hasattr(self, '_cached_fields'):
            return self._cached_fields

        result = None

        for interface, fields in self.group_fields:
            if result is None:
                result = field.Fields(interface).select(*fields)
            else:
                result += field.Fields(interface).select(*fields)

            for f in fields:
                result[f].field = copy(result[f].field)

        self._cached_fields = result
        return result

    def updateWidgets(self):
        self.update_fields()
        super(BaseGroup, self).updateWidgets()
        self.update_widgets()

    def update_fields(self):
        """ Called when it's time to make changes to the fields. """

    def update_widgets(self):
        """ Called when it's time to make changes to the widgets. """



""" Configures the email sender settings for all or just the given plone
site.

WARNING: Does not take subsites into consideration!

Run as follows:

bin/instance run <path to seantis.plonetools>
/seantis/plonetools/scripts/configure-email.py
smtp-server smtp-server-port [esmpt-username | ''] ] [esmpt-password | '']
sender-name sender-address [site]

Examples:

Setup smtp for all sites
bin/instance run <path>/configure-email.py mail.example.org 25 user@example.org
password Sender-Name user@example.org

Setup sendmail for 'my-site'
bin/instance run <path>/configure-email.py localhost 25 '' '' Sender-Name
user@example.org my-site

"""


import logging
import sys
import transaction

from plone.app.controlpanel.mail import IMailSchema
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component.hooks import setSite

log = logging.getLogger('seantis.plonetools')


def main(app):
    server = sys.argv[3]
    port = sys.argv[4]
    username = sys.argv[5]
    password = sys.argv[6]
    name = sys.argv[7]
    sender = sys.argv[8]

    if len(sys.argv) > 9:
        site = sys.argv[9]
    else:
        site = None

    sites = get_sites(app, site)
    for site in sites:
        log.info("Configuring mail for plone-site '{}'".format(site))
        configure_email(site, server, port, username, password, name, sender)

    transaction.commit()
    log.info("Done")


def get_sites(app, site):
    sites = []

    for key, item in app.items():
        if not IPloneSiteRoot.providedBy(item):
            continue

        if site is None or site == key:
            sites.append(item)

    return sites


def configure_email(site, server, port, username, password, name, sender):
    setSite(site)  # needs to be set for IMailSchema to work
    mail = IMailSchema(site)

    mail.smtp_host = unicode(server)
    mail.smtp_port = int(port)
    mail.smtp_userid = unicode(username)
    mail.smtp_pass = unicode(password)
    mail.email_from_address = unicode(sender)
    mail.email_from_name = unicode(name)


if 'app' in locals():
    main(locals()['app'])



from Products.statusmessages.interfaces import IStatusMessage
from seantis.plonetools import tools


class TranslateMixin(object):
    """ Shared by BaseView and BaseForm. """

    @property
    def domain(self):
        """ The translation domain. """
        raise NotImplementedError

    def translate(self, text, domain=None):
        return tools.translator(self.request, domain or self.domain)(text)


class StatusMessageMixin(object):

    def message(self, message, type="info"):
        IStatusMessage(self.request).add(message, type)



from setuptools import setup, find_packages
import os

name = 'seantis.plonetools'
description = (
    "A collection of tools shared with different Seantis Plone modules."
)
version = '0.15'

# needed for tests and to use the default testcase of seantis.plonetools
tests_require = [
    'plone.app.testing',
    'collective.betterbrowser'
]


def get_long_description():
    readme = open('README.rst').read()
    history = open(os.path.join('docs', 'HISTORY.rst')).read()

    # cut the part before the description to avoid repetition on pypi
    readme = readme[readme.index(description) + len(description):]

    return '\n'.join((readme, history))


setup(name=name, version=version, description=description,
      long_description=get_long_description(),
      classifiers=[
          'Framework :: Plone',
          'Framework :: Plone :: 4.3',
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Programming Language :: Python',
      ],
      keywords='plone seantis tools utilities',
      author='Seantis GmbH',
      author_email='info@seantis.ch',
      url='https://github.com/seantis/seantis.plonetools',
      license='GPL v2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['seantis'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'Plone>=4.3',
          'plone.api',
          'plone.app.dexterity',
          'plone.directives.form',
          'python-stdnum>=1.0',
          'five.grok',
          'colour',
      ],
      extras_require=dict(
          tests=tests_require
      ),
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """
      )



from logging import getLogger
log = getLogger('seantis.plonetools')

import re
import threading

from ZServer.ClockServer import ClockServer


_clockservers = dict()  # list of registered Zope Clockservers
_lock = threading.Lock()


def clear_clockservers():
    """ Clears the clockservers and connections for testing. """

    with _lock:
        for cs in _clockservers.values():
            cs.close()
        _clockservers.clear()


def register(path, period):
    """ Registers the given path with a clockserver.

    The path must be refer to the local host (think getPhysicalPath).

    Note that due to it's implementation there's no guarantee that the path
    will be called on time every time. The clockserver checks if a call is due
    every time a request comes in, or every 30 seconds when the asyncore.pollss
    path reaches it's timeout (see Lifetime/__init__.py and
    Python Stdlib/asyncore.py).

    The path is going to be called as an Anonymous user, so what can be done
    is very limited, unless seantis.plonetools.unrestricted is use.

    Although this seems to work well it's probably best not to overuse this
    feature. There might be hidden dragons (and crouching tigers).

    """

    if path not in _clockservers:
        with _lock:
            _clockservers[path] = ClockServer(
                path, period, host='localhost', logger=ClockLogger(path)
            )

    return _clockservers[path]


logexpr = re.compile(r'GET [^\s]+ HTTP/[^\s]+ ([0-9]+)')


class ClockLogger(object):

    """ Logs the clock runs by evaluating the log strings. Looks for http
    return codes to do so.

    """

    def __init__(self, path):
        self.path = path

    def return_code(self, msg):
        groups = logexpr.findall(msg)
        return groups and int(groups[0]) or None

    def log(self, msg):
        code = self.return_code(msg)

        if not code:
            log.error("ClockServer for %s returned nothing" % self.path)
        elif code == 200:
            pass
        else:
            log.warn("ClockServer for %s returned %i" % (self.path, code))
